package jalview.bin;

import java.io.File;
import java.io.IOException;
import java.lang.management.ManagementFactory;

import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

import io.github.classgraph.ClassGraph;

public class CommandsTest
{

  private static final int SETUP_TIMEOUT = 30000;

  private static class Worker extends Thread
  {
    private final Process process;

    private Integer exit;

    private Worker(Process process)
    {
      this.process = process;
    }

    @Override
    public void run()
    {
      try
      {
        exit = process.waitFor();
      } catch (InterruptedException ignore)
      {
        return;
      }
    }
  }

  private static ClassGraph scanner = null;

  private static String classpath = null;

  private static String modules = null;

  private static String java_exe = null;

  public synchronized static String getClassPath()
  {
    java_exe = System.getProperty("java.home") + File.separator + "bin"
            + File.separator + "java";
    classpath = ManagementFactory.getRuntimeMXBean().getClassPath();
    return classpath;
  }

  private Worker getJalviewDesktopRunner(boolean withAwt, String cmd,
          int timeout)
  {
    // Note: JAL-3065 - don't include quotes for lib/* because the arguments are
    // not expanded by the shell
    String classpath = getClassPath();
    String _cmd = java_exe + " "
            + (withAwt ? "-Djava.awt.headless=true" : "") + " -classpath "
            + classpath + " jalview.bin.Jalview ";
    Process ls2_proc = null;
    Worker worker = null;
    try
    {
      ls2_proc = Runtime.getRuntime().exec(_cmd + cmd);
    } catch (Throwable e1)
    {
      e1.printStackTrace();
    }
    if (ls2_proc != null)
    {
      worker = new Worker(ls2_proc);
      worker.start();
      try
      {
        worker.join(timeout);
      } catch (InterruptedException e)
      {
        System.err.println("Thread interrupted");
      }
    }
    return worker;
  }

  @BeforeTest(alwaysRun = true)
  public void initialize()
  {
    new CommandsTest();
  }

  @Test(groups = "Functional")
  public void setUpForHeadlessCommandsTest() throws IOException
  {
    String cmds = "--open=./examples/uniref50.fa";
    Worker worker = getJalviewDesktopRunner(true, cmds, SETUP_TIMEOUT);
  }

}
